@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -155,18 +155,72 @@ arena& market::create_arena ( unsigned max_num_workers, size_t stack_size ) {
     return a;
 }
 
+/** This method must be invoked under my_arenas_list_mutex. **/
 void market::detach_arena ( arena& a ) {
     __TBB_ASSERT( theMarket == this, "Global market instance was destroyed prematurely?" );
-    spin_mutex::scoped_lock lock(my_arenas_list_mutex);
 #if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
     __TBB_ASSERT( !a.my_num_workers_present, NULL );
 #endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
     __TBB_ASSERT( !a.my_slots[0].my_scheduler, NULL );
     remove_arena_from_list(a);
+    if ( a.my_aba_epoch == my_arenas_aba_epoch )
+        ++my_arenas_aba_epoch;
 }
 
+void market::try_destroy_arena ( arena* a, uintptr_t aba_epoch ) {
+    __TBB_ASSERT ( a, NULL );
+    spin_mutex::scoped_lock lock(my_arenas_list_mutex);
+    assert_market_valid();
+#if __TBB_TASK_PRIORITY
+    for ( int p = my_global_top_priority; p >= my_global_bottom_priority; --p ) {
+        priority_level_info &pl = my_priority_levels[p];
+        arena_list_type &my_arenas = pl.arenas;
+#endif /* __TBB_TASK_PRIORITY */
+        arena_list_type::iterator it = my_arenas.begin();
+        for ( ; it != my_arenas.end(); ++it ) {
+            if ( a == &*it ) {
+                if ( it->my_aba_epoch == aba_epoch ) {
+                    // Arena is alive
+                    if ( !a->my_num_workers_requested && !a->my_references ) {
+                        __TBB_ASSERT( !a->my_num_workers_allotted && (a->my_pool_state == arena::SNAPSHOT_EMPTY || !a->my_max_num_workers), "Inconsistent arena state" );
+                        // Arena is abandoned. Destroy it.
+                        detach_arena( *a );
+                        lock.release();
+                        a->free_arena();
+                    }
+                }
+                return;
+            }
+        }
+#if __TBB_TASK_PRIORITY
+    }
+#endif /* __TBB_TASK_PRIORITY */
+}
+
+void market::try_destroy_arena ( market* m, arena* a, uintptr_t aba_epoch, bool master ) {
+    // Arena may have been orphaned. Or it may have been destroyed.
+    // Thus we cannot dereference the pointer to it until its liveness is verified.
+    // Arena is alive if it is found in the market's list.
+
+    if ( m != theMarket ) {
+        // The market has already been emptied.
+        return;
+    }
+    else if ( master ) {
+        // If this is a master thread, market can be destroyed at any moment.
+        // So protect it with an extra refcount.
+        global_market_mutex_type::scoped_lock lock(theMarketMutex);
+        if ( m != theMarket )
+            return;
+        ++m->my_ref_count;
+    }
+    m->try_destroy_arena( a, aba_epoch );
+    if ( master )
+        m->release();
+}
+
+/** This method must be invoked under my_arenas_list_mutex. **/
 arena* market::arena_in_need ( arena_list_type &arenas, arena_list_type::iterator& next ) {
-    // This method is executed under my_arenas_list_mutex lock
     if ( arenas.empty() )
         return NULL;
     __TBB_ASSERT( next != arenas.end(), NULL );
@@ -176,7 +230,7 @@ arena* market::arena_in_need ( arena_list_type &arenas, arena_list_type::iterato
         if ( ++it == arenas.end() )
             it = arenas.begin();
         if ( a.num_workers_active() < a.my_num_workers_allotted ) {
-            ++a.my_num_threads_active;
+            a.my_references+=2; // add a worker
 #if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
             ++a.my_num_workers_present;
             ++my_priority_levels[a.my_top_priority].workers_present;
@@ -243,9 +297,10 @@ arena* market::arena_in_need (
         priority_level_info &pl = my_priority_levels[prev_arena->my_top_priority];
         --prev_arena->my_num_workers_present;
         --pl.workers_present;
-        if ( !--prev_arena->my_num_threads_active && !prev_arena->my_num_workers_requested ) {
+        if ( !--prev_arena->my_references && !prev_arena->my_num_workers_requested ) {
+            detach_arena( *a );
             lock.release();
-            prev_arena->close_arena();
+            a->free_arena();
             lock.acquire();
         }
     }
@@ -313,13 +368,23 @@ void market::adjust_demand ( arena& a, int delta ) {
         }
         delta = -prev_req;
     }
+#if __TBB_TASK_ARENA
+    else if ( prev_req < 0 ) {
+        delta = a.my_num_workers_requested;
+    }
+#else  /* __TBB_TASK_ARENA */
     __TBB_ASSERT( prev_req >= 0, "Part-size request to RML?" );
+#endif /* __TBB_TASK_ARENA */
 #if __TBB_TASK_PRIORITY
     intptr_t p = a.my_top_priority;
     priority_level_info &pl = my_priority_levels[p];
     pl.workers_requested += delta;
     __TBB_ASSERT( pl.workers_requested >= 0, NULL );
+#if !__TBB_TASK_ARENA
     __TBB_ASSERT( a.my_num_workers_requested >= 0, NULL );
+#else
+    //TODO: understand the assertion and modify
+#endif
     if ( a.my_num_workers_requested <= 0 ) {
         if ( a.my_top_priority != normalized_normal_priority ) {
             GATHER_STATISTIC( ++governor::local_scheduler_if_initialized()->my_counters.arena_prio_resets );
@@ -339,7 +404,11 @@ void market::adjust_demand ( arena& a, int delta ) {
         update_allotment( my_global_top_priority );
     }
     else if ( p > my_global_top_priority ) {
+#if !__TBB_TASK_ARENA
         __TBB_ASSERT( pl.workers_requested > 0, NULL );
+#else
+        //TODO: understand the assertion and modify
+#endif
         update_global_top_priority(p);
         a.my_num_workers_allotted = min( (int)my_max_num_workers, a.my_num_workers_requested );
         my_priority_levels[p - 1].workers_available = my_max_num_workers - a.my_num_workers_allotted;
@@ -384,6 +453,7 @@ void market::adjust_demand ( arena& a, int delta ) {
 
 void market::process( job& j ) {
     generic_scheduler& s = static_cast<generic_scheduler&>(j);
+    __TBB_ASSERT( governor::is_set(&s), NULL );
 #if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
     arena *a = NULL;
     while ( (a = arena_in_need(a)) )
