@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -69,17 +69,58 @@ class task_scheduler_observer_v3;
 class observer_proxy;
 #endif /* __TBB_SCHEDULER_OBSERVER */
 
-//! Cilk-style task scheduler.
-/** None of the fields here are every read or written by threads other than
+struct scheduler_state {
+    //! Index of the arena slot the scheduler occupies now, or occupied last time.
+    size_t my_arena_index;
+
+    //! Pointer to the slot in the arena we own at the moment.
+    arena_slot* my_arena_slot;
+
+    //! The arena that I own (if master) or am servicing at the moment (if worker)
+    arena* my_arena;
+
+    //! Innermost task whose task::execute() is running.
+    task* my_innermost_running_task;
+
+    //! Task, in the context of which the current TBB dispatch loop is running.
+    /** Outside of or in the outermost dispatch loop (not in a nested call to
+        wait_for_all) it is my_dummy_task for master threads, and NULL for workers. **/
+    task* my_dispatching_task;
+
+    mail_inbox my_inbox;
+
+    //! The mailbox id assigned to this scheduler.
+    /** The id is assigned upon first entry into the arena.
+        TODO: how are id's being garbage collected?
+        TODO: master thread may enter arena and leave and then reenter.
+                We want to give it the same affinity_id upon reentry, if practical.
+      */
+    affinity_id my_affinity_id;
+
+#if __TBB_SCHEDULER_OBSERVER
+    //! Last observer in the global observers list processed by this scheduler
+    observer_proxy* my_last_global_observer;
+
+    //! Last observer in the local observers list processed by this scheduler
+    observer_proxy* my_last_local_observer;
+#endif /* __TBB_SCHEDULER_OBSERVER */
+};
+
+//! Work stealing task scheduler.
+/** None of the fields here are ever read or written by threads other than
     the thread that creates the instance.
 
     Class generic_scheduler is an abstract base class that contains most of the scheduler,
     except for tweaks specific to processors and tools (e.g. VTune).
     The derived template class custom_scheduler<SchedulerTraits> fills in the tweaks. */
-class generic_scheduler: public scheduler, public ::rml::job {
+class generic_scheduler: public scheduler, public ::rml::job, private scheduler_state {
     friend class tbb::task;
     friend class market;
     friend class arena;
+#if __TBB_TASK_ARENA
+    friend class interface6::task_arena;
+    friend class interface6::wait_task;
+#endif //__TBB_TASK_ARENA
     friend class allocate_root_proxy;
     friend class governor;
 #if __TBB_TASK_GROUP_CONTEXT
@@ -108,35 +149,27 @@ class generic_scheduler: public scheduler, public ::rml::job {
 
     static const size_t null_arena_index = ~size_t(0);
 
-    //! Index of the arena slot the scheduler occupies now, or occupied last time.
-    size_t my_arena_index;
-
-    //! Capacity of the primary task pool (number of elements - pointers to task).
-    size_t my_task_pool_size;
-
-    //! Pointer to the slot in the arena we own at the moment.
-    /** When out of arena it points to this scheduler's my_dummy_slot. **/
-    mutable arena_slot* my_arena_slot;
-
     inline bool in_arena () const;
 
-    inline bool local_task_pool_empty () const;
+    inline bool is_local_task_pool_quiescent () const;
+
+    inline bool is_quiescent_local_task_pool_empty () const;
+
+    inline bool is_quiescent_local_task_pool_reset () const;
 
     //! The market I am in
     market* my_market;
 
-    //! The arena that I own (if master) or am servicing at the moment (if worker)
-    arena* my_arena;
-
     //! Random number generator used for picking a random victim from which to steal.
     FastRandom my_random;
 
+    //! Hint provided for operations with the container of starvation-resistant tasks.
+    /** Modified by the owner thread (during these operations). **/
+    unsigned hint_for_push;
+
     //! Free list of small tasks that can be reused.
     task* my_free_list;
 
-    //! Innermost task whose task::execute() is running.
-    task* my_innermost_running_task;
-
     //! Fake root task created by slave threads.
     /** The task is used as the "parent" argument to method wait_for_all. */
     task* my_dummy_task;
@@ -145,34 +178,13 @@ class generic_scheduler: public scheduler, public ::rml::job {
     /** Number of task_scheduler_init objects that point to this scheduler */
     long my_ref_count;
 
-    mail_inbox my_inbox;
-
     inline void attach_mailbox( affinity_id id );
 
-    //! The mailbox id assigned to this scheduler.
-    /** The id is assigned upon first entry into the arena.
-        TODO: how are id's being garbage collected? 
-        TODO: master thread may enter arena and leave and then reenter.
-                We want to give it the same affinity_id upon reentry, if practical.
-      */
-    affinity_id my_affinity_id;
-
     /* A couple of bools can be located here because space is otherwise just padding after my_affinity_id. */
 
     //! True if *this was created by automatic TBB initialization
     bool my_auto_initialized;
 
-#if __TBB_SCHEDULER_OBSERVER
-    //! Last observer_proxy processed by this scheduler
-    observer_proxy* my_local_last_observer_proxy;
-
-    //! Notify any entry observers that have been created since the last call by this thread.
-    void notify_entry_observers();
- 
-    //! Notify all exit observers that this thread is no longer participating in task scheduling.
-    void notify_exit_observers( bool is_worker );
-#endif /* __TBB_SCHEDULER_OBSERVER */
-
 #if __TBB_COUNT_TASK_NODES
     //! Net number of big task objects that have been allocated but not yet freed.
     intptr_t my_task_node_count;
@@ -208,15 +220,21 @@ class generic_scheduler: public scheduler, public ::rml::job {
     inline void reset_deque_and_leave_arena ( bool locked );
 
     //! Locks victim's task pool, and returns pointer to it. The pointer can be NULL.
+    /** Garbles victim_arena_slot->task_pool for the duration of the lock. **/
     task** lock_task_pool( arena_slot* victim_arena_slot ) const;
 
     //! Unlocks victim's task pool
+    /** Restores victim_arena_slot->task_pool munged by lock_task_pool. **/
     void unlock_task_pool( arena_slot* victim_arena_slot, task** victim_task_pool ) const;
 
     //! Locks the local task pool
+    /** Garbles my_arena_slot->task_pool for the duration of the lock. Requires
+        correctly set my_arena_slot->task_pool_ptr. **/
     void acquire_task_pool() const;
 
     //! Unlocks the local task pool
+    /** Restores my_arena_slot->task_pool munged by acquire_task_pool. Requires
+        correctly set my_arena_slot->task_pool_ptr. **/
     void release_task_pool() const;
 
     //! Checks if t is affinitized to another thread, and if so, bundles it as proxy.
@@ -226,6 +244,10 @@ class generic_scheduler: public scheduler, public ::rml::job {
     //! Makes newly spawned tasks visible to thieves
     inline void commit_spawned_tasks( size_t new_tail );
 
+    //! Makes relocated tasks visible to thieves and releases the local task pool.
+    /** Obviously, the task pool must be locked when calling this method. **/
+    inline void commit_relocated_tasks( size_t new_tail );
+
     //! Get a task from the local pool.
     /** Called only by the pool owner.
         Returns the pointer to the task or NULL if the pool is empty. 
@@ -233,10 +255,12 @@ class generic_scheduler: public scheduler, public ::rml::job {
     task* get_task();
 
     //! Attempt to get a task from the mailbox.
-    /** Called only by the thread that owns *this.
-        Gets a task only if there is one not yet executed by another thread.
-        If successful, unlinks the task and returns a pointer to it.
-        Otherwise returns NULL. */
+    /** Gets a task only if it has not been executed by its sender or a thief 
+        that has stolen it from the sender's task pool. Otherwise returns NULL.
+
+        This method is intended to be used only by the thread extracting the proxy 
+        from its mailbox. (In contrast to local task pool, mailbox can be read only
+        by its owner). **/
     task* get_mailbox_task();
 
     //! True if t is a task_proxy
@@ -244,10 +268,6 @@ class generic_scheduler: public scheduler, public ::rml::job {
         return t.prefix().extra_state==es_task_proxy;
     }
 
-    //! Extracts task pointer from task_proxy, and frees the proxy.
-    /** Return NULL if underlying task was claimed by mailbox. */
-    task* strip_proxy( task_proxy* result );
-
     //! Get a task from the starvation-resistant task stream of the current arena.
     /** Returns the pointer to the task, or NULL if the attempt was unsuccessful. 
         The latter case does not mean that the stream is drained, however. **/
@@ -260,21 +280,10 @@ class generic_scheduler: public scheduler, public ::rml::job {
         4 nested parallel_for calls with iteration space of 65535 grains each. **/
     static const size_t min_task_pool_size = 64;
 
-    //! Allocate task pool containing at least n elements.
-    inline task** allocate_task_pool( size_t n );
-
-    //! Deallocate task pool that was allocated by means of allocate_task_pool.
-    static void free_task_pool( task** pool ) {
-        __TBB_ASSERT( pool, "attempt to free NULL TaskPool" );
-        NFS_Free( pool );
-    }
-
     //! Makes sure that the task pool can accommodate at least n more elements
-    /** If necessary relocates existing task pointers or grows the ready task deque. **/
-    inline void prepare_task_pool( size_t n );
-
-    //! Grow ready task deque to at least n elements.
-    void grow_task_pool( size_t n );
+    /** If necessary relocates existing task pointers or grows the ready task deque.
+        Returns (possible updated) tail index (not accounting for n). **/
+    size_t prepare_task_pool( size_t n );
 
     //! Initialize a scheduler for a master thread.
     static generic_scheduler* create_master( arena& a );
@@ -286,7 +295,7 @@ class generic_scheduler: public scheduler, public ::rml::job {
     static generic_scheduler* create_worker( market& m, size_t index );
 
     //! Perform necessary cleanup when a worker thread finishes.
-    static void cleanup_worker( void* arg, bool is_worker );
+    static void cleanup_worker( void* arg, bool worker );
 
 protected:
     generic_scheduler( arena*, size_t index );
@@ -295,27 +304,19 @@ protected:
     //! Check that internal data structures are in consistent state.
     /** Raises __TBB_ASSERT failure if inconsistency is found. */
     void assert_task_pool_valid () const;
-    static void fill_with_canary_pattern ( task** task_pool, size_t first, size_t last );
 #else
     void assert_task_pool_valid() const {}
-    static void fill_with_canary_pattern ( task**, size_t, size_t ) {}
 #endif /* TBB_USE_ASSERT <= 1 */
 
 public:
     /*override*/ 
-    inline void spawn( task& first, task*& next );
+    void spawn( task& first, task*& next );
 
     /*override*/ 
-    inline void spawn_root_and_wait( task& first, task*& next );
+    void spawn_root_and_wait( task& first, task*& next );
 
     /*override*/ 
-    inline void enqueue( task&, void* reserved );
-
-#if __TBB_TASK_PRIORITY
-    void local_enqueue( task&, priority_t prio );
-#else /* !__TBB_TASK_PRIORITY */
-    void local_enqueue( task& );
-#endif /* !__TBB_TASK_PRIORITY */
+    void enqueue( task&, void* reserved );
 
     void local_spawn( task& first, task*& next );
     void local_spawn_root_and_wait( task& first, task*& next );
@@ -334,8 +335,6 @@ public:
     template<free_task_hint h>
     void free_task( task& t );
 
-    inline void free_task_proxy( task_proxy& tp );
-
     //! Return task object to the memory allocator.
     inline void deallocate_task( task& t );
 
@@ -374,15 +373,16 @@ public:
     //! List of small tasks that have been returned to this scheduler by other schedulers.
     task* my_return_list;
 
-    //! Try getting a task from the mailbox or stealing from another scheduler.
-    /** Redirects to a customization. */
-    virtual task* receive_or_steal_task( reference_count&, bool ) = 0; 
+    //! Try getting a task from other threads (via mailbox, stealing, FIFO queue, orphans adoption).
+    /** Returns obtained task or NULL if all attempts fail. */
+    virtual task* receive_or_steal_task( __TBB_atomic reference_count& completion_ref_count,
+                                         bool return_if_no_work ) = 0;
 
     //! Free a small task t that that was allocated by a different scheduler 
     void free_nonlocal_small_task( task& t ); 
 
 #if __TBB_TASK_GROUP_CONTEXT
-    //! Padding isolating thread local members from members that can be written to by other threads.
+    //! Padding isolating thread-local members from members that can be written to by other threads.
     char _padding1[NFS_MaxLineSize - sizeof(context_list_node_t)];
 
     //! Head of the thread specific list of task group contexts.
@@ -404,7 +404,7 @@ public:
     /** Together with my_nonlocal_ctx_list_update constitute synchronization protocol
         that keeps hot path of context destruction (by the owner thread) mostly 
         lock-free. **/
-    uintptr_t my_local_ctx_list_update;
+    tbb::atomic<uintptr_t> my_local_ctx_list_update;
 
 #if __TBB_TASK_PRIORITY
     //! True if the scheduler is on the outermost dispatch level in a worker thread.
@@ -413,16 +413,11 @@ public:
     //! Returns reference priority used to decide whether a task should be offloaded.
     inline intptr_t effective_reference_priority () const;
 
-    // Task, in the context of which the current TBB dispatch loop is running.
-    /** Outside of or in the outermost dispatch loop (not in a nested call to
-        wait_for_all) it is my_dummy_task for master threads, and NULL for workers. **/
-    task* my_dispatching_task;
-
     //! Latest known highest priority of tasks in the market or arena.
     /** Master threads currently tracks only tasks in their arenas, while workers
         take into account global top priority (among all arenas in the market). **/
     volatile intptr_t *my_ref_top_priority;
-    
+
     //! Task pool for offloading tasks with priorities lower than the current top priority.
     task* my_offloaded_tasks;
 
@@ -468,22 +463,16 @@ private:
     ::rml::server::execution_resource_t master_exec_resource;
 #endif /* _WIN32||_WIN64 */
 
-    //! Dummy slot used when scheduler is not in arena
-    /** The data structure is heavily padded, therefore it should be placed after 
-        other data fields used by the owner thread only to allow compiler using 
-        instructions with short offsets when accessing the majority of data members. **/
-    arena_slot my_dummy_slot;
-
 #if __TBB_TASK_GROUP_CONTEXT
     //! Flag indicating that a context is being destructed by non-owner thread.
     /** See also my_local_ctx_list_update. **/
-    uintptr_t my_nonlocal_ctx_list_update;
+    tbb::atomic<uintptr_t> my_nonlocal_ctx_list_update;
 #endif /* __TBB_TASK_GROUP_CONTEXT */
 
 #if __TBB_SURVIVE_THREAD_SWITCH
     __cilk_tbb_unwatch_thunk my_cilk_unwatch_thunk;
 #if TBB_USE_ASSERT
-    //! State values used to check interface contract with Cilk runtime.
+    //! State values used to check interface contract with cilkrts.
     /** Names of cs_running...cs_freed derived from state machine diagram in cilk-tbb-interop.h */
     enum cilk_state_t {
         cs_none=0xF000, // Start at nonzero value so that we can detect use of zeroed memory.
@@ -514,32 +503,26 @@ private:
 namespace tbb {
 namespace internal {
 
-inline void tbb::internal::generic_scheduler::spawn( task& first, task*& next ) {
-    governor::local_scheduler()->local_spawn( first, next );
-}
-
-inline void tbb::internal::generic_scheduler::spawn_root_and_wait( task& first, task*& next ) {
-    governor::local_scheduler()->local_spawn_root_and_wait( first, next );
-}
-
-inline void tbb::internal::generic_scheduler::enqueue( task& t, void* prio ) {
-    governor::local_scheduler()->local_enqueue( t
-#if __TBB_TASK_PRIORITY
-                                                , (priority_t)(intptr_t)prio
-#endif /* __TBB_TASK_PRIORITY */
-                                              );
-}
-
 inline bool generic_scheduler::in_arena () const {
-    return my_arena_slot != &my_dummy_slot;
+    return my_arena_slot->task_pool != EmptyTaskPool;
 }
 
-inline bool generic_scheduler::local_task_pool_empty () const {
-    return my_arena_slot->task_pool == EmptyTaskPool || my_arena_slot->head >= my_arena_slot->tail;
+inline bool generic_scheduler::is_local_task_pool_quiescent () const {
+    return my_arena_slot->task_pool == EmptyTaskPool || my_arena_slot->task_pool == LockedTaskPool;
+}
+
+inline bool generic_scheduler::is_quiescent_local_task_pool_empty () const {
+    __TBB_ASSERT( is_local_task_pool_quiescent(), "Task pool is not quiescent" );
+    return __TBB_load_relaxed(my_arena_slot->head) == __TBB_load_relaxed(my_arena_slot->tail);
+}
+
+inline bool generic_scheduler::is_quiescent_local_task_pool_reset () const {
+    __TBB_ASSERT( is_local_task_pool_quiescent(), "Task pool is not quiescent" );
+    return __TBB_load_relaxed(my_arena_slot->head) == 0 && __TBB_load_relaxed(my_arena_slot->tail) == 0;
 }
 
 inline bool generic_scheduler::master_outermost_level () const {
-    return my_innermost_running_task == my_dummy_task;
+    return my_dispatching_task == my_dummy_task;
 }
 
 #if __TBB_TASK_GROUP_CONTEXT
@@ -555,23 +538,14 @@ inline void generic_scheduler::attach_mailbox( affinity_id id ) {
 }
 
 inline bool generic_scheduler::is_worker() {
-    return my_arena_index != 0;
+    return my_arena_index != 0; //TODO: rework for multiple master
 }
 
 inline unsigned generic_scheduler::number_of_workers_in_my_arena() {
     return my_arena->my_max_num_workers;
 }
 
-inline void generic_scheduler::free_task_proxy( task_proxy& tp ) {
-#if TBB_USE_ASSERT
-    poison_pointer( tp.outbox );
-    poison_pointer( tp.next_in_mailbox );
-    tp.task_and_tag = 0xDEADBEEF;
-#endif /* TBB_USE_ASSERT */
-    free_task<small_task>(tp);
-}
-
-    //! Return task object to the memory allocator.
+//! Return task object to the memory allocator.
 inline void generic_scheduler::deallocate_task( task& t ) {
 #if TBB_USE_ASSERT
     task_prefix& p = t.prefix();
@@ -594,52 +568,31 @@ inline intptr_t generic_scheduler::get_task_node_count( bool count_arena_workers
 inline void generic_scheduler::reset_deque_and_leave_arena ( bool locked ) {
     if ( !locked )
         acquire_task_pool();
-    my_arena_slot->tail = my_arena_slot->head = 0;
+    __TBB_store_relaxed( my_arena_slot->tail, 0 );
+    __TBB_store_relaxed( my_arena_slot->head, 0 );
     leave_arena();
 }
 
-inline task** generic_scheduler::allocate_task_pool( size_t n ) {
-    size_t byte_size = ((n * sizeof(task*) + NFS_MaxLineSize - 1) / NFS_MaxLineSize) * NFS_MaxLineSize;
-    my_task_pool_size = byte_size / sizeof(task*);
-    task** new_pool = (task**)NFS_Allocate( byte_size, 1, NULL );
-    // No need to clear the fresh deque since valid items are designated by the head and tail members.
-    // But fill it with a canary pattern in the high vigilance debug mode.
-    fill_with_canary_pattern( new_pool, 0, my_task_pool_size );
-    return new_pool;
-}
-
-inline void generic_scheduler::prepare_task_pool( size_t num_tasks ) {
-    if ( my_arena_slot->tail + num_tasks <= my_task_pool_size )
-        return;
-    // 1 compensates for head possibly temporarily incremented by a thief
-    size_t new_size = my_arena_slot->tail - my_arena_slot->head + num_tasks + 1;
-    // If the free space at the beginning of the task pool is too short, we
-    // are likely facing a pathological single-producer-multiple-consumers
-    // scenario, and thus it's better to expand the task pool
-    if ( new_size <= my_task_pool_size - min_task_pool_size/4 ) {
-        // Relocate the busy part to the deque beginning
-        acquire_task_pool();
-        size_t new_tail = my_arena_slot->tail - my_arena_slot->head;
-        memmove( my_dummy_slot.task_pool, my_dummy_slot.task_pool + my_arena_slot->head, new_tail * sizeof(task*) );
-        fill_with_canary_pattern( my_dummy_slot.task_pool, new_tail, my_arena_slot->tail );
-        my_arena_slot->head = 0;
-        // Tail is updated last to minimize probability of a thread making arena 
-        // snapshot being misguided into thinking that this task pool is empty.
-        my_arena_slot->tail = new_tail;
-        release_task_pool();
-    }
-    else
-        grow_task_pool( my_task_pool_size + num_tasks );
-}
-
+//TODO: move to arena_slot
 inline void generic_scheduler::commit_spawned_tasks( size_t new_tail ) {
-    __TBB_ASSERT ( new_tail <= my_task_pool_size, "task deque end was overwritten" );
-    ITT_NOTIFY(sync_releasing, my_arena_slot);
+    __TBB_ASSERT ( new_tail <= my_arena_slot->my_task_pool_size, "task deque end was overwritten" );
+    // emit "task was released" signal
+    ITT_NOTIFY(sync_releasing, (void*)((uintptr_t)my_arena_slot+sizeof(uintptr_t)));
     // Release fence is necessary to make sure that previously stored task pointers
     // are visible to thieves.
     __TBB_store_with_release( my_arena_slot->tail, new_tail );
 }
 
+void generic_scheduler::commit_relocated_tasks ( size_t new_tail ) {
+    __TBB_ASSERT( is_local_task_pool_quiescent(),
+                  "Task pool must be locked when calling commit_relocated_tasks()" );
+    __TBB_store_relaxed( my_arena_slot->head, 0 );
+    // Tail is updated last to minimize probability of a thread making arena 
+    // snapshot being misguided into thinking that this task pool is empty.
+    __TBB_store_relaxed( my_arena_slot->tail, new_tail );
+    release_task_pool();
+}
+
 template<free_task_hint h>
 void generic_scheduler::free_task( task& t ) {
     GATHER_STATISTIC(--my_counters.active_tasks);
@@ -647,11 +600,9 @@ void generic_scheduler::free_task( task& t ) {
     // Verify that optimization hints are correct.
     __TBB_ASSERT( h!=small_local_task || p.origin==this, NULL );
     __TBB_ASSERT( !(h&small_task) || p.origin, NULL );
-#if TBB_USE_ASSERT
-    p.depth = 0xDEADBEEF;
-    p.ref_count = 0xDEADBEEF;
+    poison_value(p.depth);
+    poison_value(p.ref_count);
     poison_pointer(p.owner);
-#endif /* TBB_USE_ASSERT */
     __TBB_ASSERT( 1L<<t.state() & (1L<<task::executing|1L<<task::allocated), NULL );
     p.state = task::freed;
     if( h==small_local_task || p.origin==this ) {
@@ -683,7 +634,7 @@ inline intptr_t generic_scheduler::effective_reference_priority () const {
             ? *my_ref_top_priority : my_arena->my_top_priority;
 }
 
-inline void generic_scheduler::offload_task ( task& t, intptr_t p ) {
+inline void generic_scheduler::offload_task ( task& t, intptr_t /*priority*/ ) {
     GATHER_STATISTIC( ++my_counters.prio_tasks_offloaded );
     __TBB_ASSERT( my_offloaded_task_list_tail_link && !*my_offloaded_task_list_tail_link, NULL );
 #if TBB_USE_ASSERT
