@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -37,6 +37,7 @@
 #define tbb_tests_harness_H
 
 #include "tbb/tbb_config.h"
+#include "harness_defs.h"
 
 namespace Harness {
     enum TestResult {
@@ -50,7 +51,7 @@ namespace Harness {
 /** It MUST be defined by the test application.
 
     If HARNESS_NO_PARSE_COMMAND_LINE macro was not explicitly set before including harness.h,
-    then global variables Verbose, MinThread, and MaxThread will be available and
+    then global variables MinThread, and MaxThread will be available and
     initialized when it is called.
 
     Returns Harness::Done when the tests passed successfully. When the test fail, it must
@@ -61,14 +62,6 @@ namespace Harness {
     before including harness.h **/
 int TestMain ();
 
-#define __TBB_LAMBDAS_PRESENT  ( _MSC_VER >= 1600 && !__INTEL_COMPILER || __INTEL_COMPILER > 1100 && _TBB_CPP0X )
-
-#if defined(_MSC_VER) && _MSC_VER < 1400
-    #define __TBB_EXCEPTION_TYPE_INFO_BROKEN 1
-#else
-    #define __TBB_EXCEPTION_TYPE_INFO_BROKEN 0
-#endif
-
 #if __SUNPRO_CC
     #include <stdlib.h>
     #include <string.h>
@@ -87,14 +80,21 @@ int TestMain ();
 
 #include <new>
 
+#if __TBB_MIC_NATIVE
+    #include "harness_mic.h"
+#else
     #define HARNESS_EXPORT
     #define REPORT_FATAL_ERROR REPORT
+#endif /* !__MIC__ */
 
 #if _WIN32||_WIN64
     #include "tbb/machine/windows_api.h"
 #if _XBOX
     #undef HARNESS_NO_PARSE_COMMAND_LINE
     #define HARNESS_NO_PARSE_COMMAND_LINE 1
+#endif
+#if __TBB_WIN8UI_SUPPORT
+#include <thread>
 #endif
     #include <process.h>
 #else
@@ -117,6 +117,7 @@ void SetHarnessErrorProcessing( test_error_extra_t extra_call ) {
     ErrorExtraCall = extra_call;
     // TODO: add tbb::set_assertion_handler(ReportError);
 }
+
 //! Reports errors issued by failed assertions
 void ReportError( const char* filename, int line, const char* expression, const char * message ) {
 #if __TBB_ICL_11_1_CODE_GEN_BROKEN
@@ -130,6 +131,8 @@ void ReportError( const char* filename, int line, const char* expression, const
     TerminateProcess(GetCurrentProcess(), 1);
 #elif HARNESS_EXIT_ON_ASSERT
     exit(1);
+#elif HARNESS_CONTINUE_ON_ASSERT
+    // continue testing
 #else
     abort();
 #endif /* HARNESS_EXIT_ON_ASSERT */
@@ -138,10 +141,30 @@ void ReportError( const char* filename, int line, const char* expression, const
 void ReportWarning( const char* filename, int line, const char* expression, const char * message ) {
     REPORT("Warning: %s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
 }
-#else
-#define ASSERT(p,msg) ((void)0)
-#define ASSERT_WARNING(p,msg) ((void)0)
-#endif /* HARNESS_NO_ASSERT */
+#else /* !HARNESS_NO_ASSERT */
+//! Utility template function to prevent "unused" warnings by various compilers.
+template<typename T> void suppress_unused_warning( const T& ) {}
+
+#define ASSERT(p,msg) (suppress_unused_warning(p), (void)0)
+#define ASSERT_WARNING(p,msg) (suppress_unused_warning(p), (void)0)
+#endif /* !HARNESS_NO_ASSERT */
+
+template<typename T, size_t N>
+inline size_t array_length(const T(&)[N])
+{
+   return N;
+}
+
+//TODO: remove this #if __TBB_INITIALIZER_LISTS_PRESENT
+//it looks like all other compilers except gcc issue warnings/errors then they see
+//declaration of zero sized array
+#if __TBB_INITIALIZER_LISTS_PRESENT
+template<typename T>
+inline size_t array_length(const T[0])
+{
+   return 0;
+}
+#endif //__TBB_INITIALIZER_LISTS_PRESENT
 
 #if !HARNESS_NO_PARSE_COMMAND_LINE
 
@@ -223,21 +246,71 @@ static void ParseCommandLine( int argc, char* argv[] ) {
 
 #if !HARNESS_CUSTOM_MAIN
 
+#if __TBB_MPI_INTEROP
+#undef SEEK_SET
+#undef SEEK_CUR
+#undef SEEK_END
+#include "mpi.h"
+#endif
 
 HARNESS_EXPORT
 #if HARNESS_NO_PARSE_COMMAND_LINE
 int main() {
+#if __TBB_MPI_INTEROP
+    MPI_Init(NULL,NULL); 
+#endif
 #else
 int main(int argc, char* argv[]) {
     ParseCommandLine( argc, argv );
+#if __TBB_MPI_INTEROP
+    MPI_Init(&argc,&argv); 
 #endif
+#endif
+#if __TBB_MPI_INTEROP
+    // Simple TBB/MPI interoperability harness for most of tests
+    // Worker processes send blocking messages to the master process about their rank and group size
+    // Master process receives this info and print it in verbose mode
+    int rank, size, myrank;
+    MPI_Status status;
+    MPI_Comm_size(MPI_COMM_WORLD,&size); 
+    MPI_Comm_rank(MPI_COMM_WORLD,&myrank); 
+    if (myrank == 0) {
+#if !HARNESS_NO_PARSE_COMMAND_LINE
+        REMARK("Hello mpi world. I am %d of %d\n", myrank, size);
+#endif
+        for ( int i = 1; i < size; i++ ) {
+            MPI_Recv (&rank, 1, MPI_INT, i, 1, MPI_COMM_WORLD, &status);
+            MPI_Recv (&size, 1, MPI_INT, i, 1, MPI_COMM_WORLD, &status);
+#if !HARNESS_NO_PARSE_COMMAND_LINE
+            REMARK("Hello mpi world. I am %d of %d\n", rank, size);
+#endif
+        }
+    } else {
+        MPI_Send (&myrank, 1, MPI_INT, 0, 1, MPI_COMM_WORLD);
+        MPI_Send (&size, 1, MPI_INT, 0, 1, MPI_COMM_WORLD);
+    }
+#endif
+#if __TBB_MIC
+    int res = Harness::Unknown;
+    #pragma offload target(mic:-1) out(res)
+    {
+        res = TestMain ();
+    }
+#else
     int res = TestMain ();
+#endif
     ASSERT( res==Harness::Done || res==Harness::Skipped, "Wrong return code by TestMain");
+#if __TBB_MPI_INTEROP
+    if (myrank == 0) {
+        REPORT( res==Harness::Done ? "done\n" : "skip\n" );
+    }
+    MPI_Finalize();
+#else
     REPORT( res==Harness::Done ? "done\n" : "skip\n" );
+#endif
     return 0;
 }
 
-
 #endif /* !HARNESS_CUSTOM_MAIN */
 
 //! Base class for prohibiting compiler-generated operator=
@@ -283,7 +356,8 @@ public:
         // launched by make, the default stack size is set to the hard limit, and
         // calls to pthread_create fail with out-of-memory error.
         // Therefore we set the stack size explicitly (as for TBB worker threads).
-        const size_t MByte = 1<<20;
+// TODO: make a single definition of MByte used by all tests.
+        const size_t MByte = 1024*1024;
 #if __i386__||__i386
         const size_t stack_size = 1*MByte;
 #elif __x86_64__
@@ -308,12 +382,15 @@ public:
     //! Wait for task to finish
     void wait_to_finish() {
 #if _WIN32||_WIN64
-        DWORD status = WaitForSingleObject( thread_handle, INFINITE );
+        DWORD status = WaitForSingleObjectEx( thread_handle, INFINITE, FALSE );
         ASSERT( status!=WAIT_FAILED, "WaitForSingleObject failed" );
         CloseHandle( thread_handle );
 #else
         int status = pthread_join( thread_id, NULL );
         ASSERT( !status, "pthread_join failed" );
+#endif
+#if HARNESS_NO_ASSERT
+        (void)status;
 #endif
     }
 
@@ -457,7 +534,15 @@ public:
 #endif /* !HARNESS_NO_ASSERT */
 
 #if _WIN32 || _WIN64
-    void Sleep ( int ms ) { ::Sleep(ms); }
+    void Sleep ( int ms ) { 
+#if !__TBB_WIN8UI_SUPPORT
+        ::Sleep(ms); 
+#else
+         std::chrono::milliseconds sleep_time( ms );
+         std::this_thread::sleep_for( sleep_time );
+#endif
+
+    }
 
     typedef DWORD tid_t;
     tid_t CurrentTid () { return GetCurrentThreadId(); }
