@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,20 +26,12 @@
     the GNU General Public License.
 */
 
-#include "tbb/task.h"
+#include "harness_task.h"
 #include "tbb/atomic.h"
 #include "tbb/tbb_thread.h"
-#include "harness_assert.h"
+#include "tbb/task_scheduler_init.h"
 #include <cstdlib>
 
-//------------------------------------------------------------------------
-// Helper for verifying that old use cases of spawn syntax still work.
-//------------------------------------------------------------------------
-tbb::task* GetTaskPtr( int& counter ) {
-    ++counter;
-    return NULL;
-}
- 
 //------------------------------------------------------------------------
 // Test for task::spawn_children and task_list
 //------------------------------------------------------------------------
@@ -48,11 +40,11 @@ class UnboundedlyRecursiveOnUnboundedStealingTask : public tbb::task {
     typedef UnboundedlyRecursiveOnUnboundedStealingTask this_type;
 
     this_type *m_Parent;
-    const int m_Depth; 
+    const int m_Depth;
     volatile bool m_GoAhead;
 
     // Well, virtually unboundedly, for any practical purpose
-    static const int max_depth = 1000000; 
+    static const int max_depth = 1000000;
 
 public:
     UnboundedlyRecursiveOnUnboundedStealingTask( this_type *parent_ = NULL, int depth_ = max_depth )
@@ -63,12 +55,12 @@ public:
 
     /*override*/
     tbb::task* execute() {
-        // Using large padding array sppeds up reaching stealing limit
+        // Using large padding array speeds up reaching stealing limit
         const int paddingSize = 16 * 1024;
         volatile char padding[paddingSize];
         if( !m_Parent || (m_Depth > 0 &&  m_Parent->m_GoAhead) ) {
             if ( m_Parent ) {
-                // We are stolen, let our parent to start waiting for us
+                // We are stolen, let our parent start waiting for us
                 m_Parent->m_GoAhead = false;
             }
             tbb::task &t = *new( allocate_child() ) this_type(this, m_Depth - 1);
@@ -79,7 +71,7 @@ public:
                 ++padding[i % paddingSize];
                 __TBB_Yield();
             }
-            // If our child has not been stolen yet, then prohibit it siring ones 
+            // If our child has not been stolen yet, then prohibit it siring ones
             // of its own (when this thread executes it inside the next wait_for_all)
             m_GoAhead = false;
             wait_for_all();
@@ -92,7 +84,7 @@ tbb::atomic<int> Count;
 
 class RecursiveTask: public tbb::task {
     const int m_ChildCount;
-    const int m_Depth; 
+    const int m_Depth;
     //! Spawn tasks in list.  Exact method depends upon m_Depth&bit_mask.
     void SpawnList( tbb::task_list& list, int bit_mask ) {
         if( m_Depth&bit_mask ) {
@@ -133,14 +125,15 @@ static int Expected( int child_count, int depth ) {
     return depth<=0 ? 1 : 1+child_count*Expected(child_count/2,depth-1);
 }
 
-#include "tbb/task_scheduler_init.h"
-#include "harness.h"
-
 void TestStealLimit( int nthread ) {
+#if __TBB_DEFINE_MIC
+    REMARK( "skipping steal limiting heuristics for %d threads\n", nthread );
+#else// !_TBB_DEFINE_MIC
     REMARK( "testing steal limiting heuristics for %d threads\n", nthread );
     tbb::task_scheduler_init init(nthread);
     tbb::task &t = *new( tbb::task::allocate_root() ) UnboundedlyRecursiveOnUnboundedStealingTask();
     tbb::task::spawn_root_and_wait(t);
+#endif// _TBB_DEFINE_MIC
 }
 
 //! Test task::spawn( task_list& )
@@ -163,46 +156,19 @@ void TestSpawnRootList( int nthread ) {
     for( int j=0; j<5; ++j )
         for( int k=0; k<10; ++k ) {
             Count = 0;
-            tbb::task_list list; 
+            tbb::task_list list;
             for( int i=0; i<k; ++i )
                 list.push_back( *new( tbb::task::allocate_root() ) RecursiveTask(j,4) );
             tbb::task::spawn_root_and_wait(list);
             int expected = k*Expected(j,4);
             ASSERT( Count==expected, NULL );
-        }    
+        }
 }
 
 //------------------------------------------------------------------------
 // Test for task::recycle_as_safe_continuation
 //------------------------------------------------------------------------
 
-class TaskGenerator: public tbb::task {
-    int m_ChildCount;
-    int m_Depth;
-    
-public:
-    TaskGenerator( int child_count, int _depth ) : m_ChildCount(child_count), m_Depth(_depth) {}
-    ~TaskGenerator( ) { m_ChildCount = m_Depth = -125; }
-
-    /*override*/ tbb::task* execute() {
-        ASSERT( m_ChildCount>=0 && m_Depth>=0, NULL );
-        if( m_Depth>0 ) {
-            recycle_as_safe_continuation();
-            set_ref_count( m_ChildCount+1 );
-            int k=0; 
-            for( int j=0; j<m_ChildCount; ++j ) {
-                tbb::task& t = *new( allocate_child() ) TaskGenerator(m_ChildCount/2,m_Depth-1);
-                GetTaskPtr(k)->spawn(t);
-            }
-            ASSERT(k==m_ChildCount,NULL);
-            --m_Depth;
-            __TBB_Yield();
-            ASSERT( state()==recycle && ref_count()>0, NULL);
-        }
-        return NULL;
-    }
-};
-
 void TestSafeContinuation( int nthread ) {
     REMARK("testing task::recycle_as_safe_continuation for %d threads\n",nthread);
     tbb::task_scheduler_init init(nthread);
@@ -218,18 +184,18 @@ void TestSafeContinuation( int nthread ) {
 tbb::atomic<int> TotalCount;
 
 struct AffinityTask: public tbb::task {
-    const affinity_id expected_affinity_id; 
+    const affinity_id expected_affinity_id;
     bool noted;
     /** Computing affinities is NOT supported by TBB, and may disappear in the future.
         It is done here for sake of unit testing. */
-    AffinityTask( int expected_affinity_id_ ) : 
-        expected_affinity_id(affinity_id(expected_affinity_id_)), 
-        noted(false) 
+    AffinityTask( int expected_affinity_id_ ) :
+        expected_affinity_id(affinity_id(expected_affinity_id_)),
+        noted(false)
     {
         set_affinity(expected_affinity_id);
-        ASSERT( 0u-expected_affinity_id>0u, "affinity_id not an unsigned integral type?" );  
+        ASSERT( 0u-expected_affinity_id>0u, "affinity_id not an unsigned integral type?" );
         ASSERT( affinity()==expected_affinity_id, NULL );
-    } 
+    }
     /*override*/ tbb::task* execute() {
         ++TotalCount;
         return NULL;
@@ -251,7 +217,7 @@ struct AffinityTask: public tbb::task {
 void TestAffinity( int nthread ) {
     TotalCount = 0;
     int n = tbb::task_scheduler_init::default_num_threads();
-    if( n>nthread ) 
+    if( n>nthread )
         n = nthread;
     tbb::task_scheduler_init init(n);
     tbb::empty_task* t = new( tbb::task::allocate_root() ) tbb::empty_task;
@@ -260,11 +226,11 @@ void TestAffinity( int nthread ) {
     // Set ref_count for n-1 children, plus 1 for the wait.
     t->set_ref_count(n);
     // Spawn n-1 affinitized children.
-    for( int i=1; i<n; ++i ) 
+    for( int i=1; i<n; ++i )
         tbb::task::spawn( *new(t->allocate_child()) AffinityTask(i) );
     if( n>1 ) {
         // Keep master from stealing
-        while( TotalCount!=n-1 ) 
+        while( TotalCount!=n-1 )
             __TBB_Yield();
     }
     // Wait for the children
@@ -292,16 +258,16 @@ struct NoteAffinityTask: public tbb::task {
     }
 };
 
-// This test checks one of the paths inside the scheduler by affinitizing the child task 
-// to non-existent thread so that it is proxied in the local task pool but not retrieved 
-// by another thread. 
+// This test checks one of the paths inside the scheduler by affinitizing the child task
+// to non-existent thread so that it is proxied in the local task pool but not retrieved
+// by another thread.
 // If no workers requested, the extra slot #2 is allocated for a worker thread to serve
 // "enqueued" tasks. In this test, it is used only for the affinity purpose.
 void TestNoteAffinityContext() {
     tbb::task_scheduler_init init(1);
     tbb::empty_task* t = new( tbb::task::allocate_root() ) tbb::empty_task;
     t->set_ref_count(2);
-    // This master in the absence of workers will have an affinity id of 1. 
+    // This master in the absence of workers will have an affinity id of 1.
     // So use another number to make the task get proxied.
     tbb::task::spawn( *new(t->allocate_child()) NoteAffinityTask(2) );
     t->wait_for_all();
@@ -325,7 +291,7 @@ struct ConstructionFailure {
     #pragma warning (disable: 4702)
 #endif
 
-//! Task that cannot be constructed.  
+//! Task that cannot be constructed.
 template<size_t N>
 struct UnconstructibleTask: public tbb::empty_task {
     char space[N];
@@ -385,14 +351,14 @@ void TestUnconstructibleTask() {
 
 //! Task with members of type T.
 /** The task recursively creates tasks. */
-template<typename T> 
+template<typename T>
 class TaskWithMember: public tbb::task {
     T x;
     T y;
     unsigned char count;
     /*override*/ tbb::task* execute() {
         x = y;
-        if( count>0 ) { 
+        if( count>0 ) {
             set_ref_count(2);
             tbb::task* t = new( allocate_child() ) TaskWithMember<T>(count-1);
             spawn_and_wait_for_all(*t);
@@ -407,7 +373,7 @@ public:
     #pragma warning (pop)
 #endif
 
-template<typename T> 
+template<typename T>
 void TestAlignmentOfOneClass() {
     typedef TaskWithMember<T> task_type;
     tbb::task* t = new( tbb::task::allocate_root() ) task_type(10);
@@ -429,7 +395,10 @@ void TestAlignment() {
     TestAlignmentOfOneClass<double>();
 #if HAVE_m128
     TestAlignmentOfOneClass<__m128>();
-#endif /* HAVE_m128 */
+#endif
+#if HAVE_m256
+    if (have_AVX()) TestAlignmentOfOneClass<__m256>();
+#endif
 }
 
 //------------------------------------------------------------------------
@@ -445,7 +414,7 @@ struct RightFibTask: public tbb::task {
     task* execute() {
         *y = Fib(n-1);
         return 0;
-    } 
+    }
 };
 
 int Fib( int n ) {
@@ -453,7 +422,7 @@ int Fib( int n ) {
         return n;
     } else {
         // y actually does not need to be initialized.  It is initialized solely to suppress
-        // a gratuitous warning "potentially uninitialized local variable". 
+        // a gratuitous warning "potentially uninitialized local variable".
         int y=-1;
         tbb::task* root_task = new( tbb::task::allocate_root() ) tbb::empty_task;
         root_task->set_ref_count(2);
@@ -468,7 +437,7 @@ int Fib( int n ) {
 void TestLeftRecursion( int p ) {
     REMARK("testing non-spawned roots for %d threads\n",p);
     tbb::task_scheduler_init init(p);
-    int sum = 0; 
+    int sum = 0;
     for( int i=0; i<100; ++i )
         sum +=Fib(10);
     ASSERT( sum==5500, NULL );
@@ -484,9 +453,9 @@ class DagTask: public tbb::task {
     number_t sum_from_left, sum_from_above;
     void check_sum( number_t sum ) {
         number_t expected_sum = 1;
-        for( int k=i+1; k<=i+j; ++k ) 
+        for( int k=i+1; k<=i+j; ++k )
             expected_sum *= k;
-        for( int k=1; k<=j; ++k ) 
+        for( int k=1; k<=j; ++k )
             expected_sum /= k;
         ASSERT(sum==expected_sum, NULL);
     }
@@ -506,11 +475,11 @@ public:
         }
         if( DagTask* t = successor_to_below ) {
             t->sum_from_above = sum;
-            if( t->decrement_ref_count()==0 ) 
+            if( t->decrement_ref_count()==0 )
                 // Test using bypass to evaluate DAG
                 return t;
-        } 
-        return NULL;  
+        }
+        return NULL;
     }
     ~DagTask() {++destruction_count;}
     static tbb::atomic<int> execution_count;
@@ -527,10 +496,10 @@ void TestDag( int p ) {
     DagTask::destruction_count=0;
     const int n = 10;
     DagTask* a[n][n];
-    for( int i=0; i<n; ++i ) 
+    for( int i=0; i<n; ++i )
         for( int j=0; j<n; ++j )
             a[i][j] = new( tbb::task::allocate_root() ) DagTask(i,j);
-    for( int i=0; i<n; ++i ) 
+    for( int i=0; i<n; ++i )
         for( int j=0; j<n; ++j ) {
             a[i][j]->successor_to_below = i+1<n ? a[i+1][j] : NULL;
             a[i][j]->successor_to_right = j+1<n ? a[i][j+1] : NULL;
@@ -625,9 +594,8 @@ void TestUserThread( int p ) {
     }
 }
 
-
 class TaskWithChildToSteal : public tbb::task {
-    const int m_Depth; 
+    const int m_Depth;
     volatile bool m_GoAhead;
 
 public:
@@ -657,6 +625,7 @@ public:
     }
 }; // TaskWithChildToSteal
 
+// Success criterion of this test is not hanging
 void TestDispatchLoopResponsiveness() {
     REMARK("testing that dispatch loops do not go into eternal sleep when all remaining children are stolen\n");
     // Recursion depth values test the following sorts of dispatch loops
@@ -670,7 +639,6 @@ void TestDispatchLoopResponsiveness() {
         t.SpawnAndWaitOnParent();
     }
     r.destroy(r);
-    // The success criteria of this test is not hanging
 }
 
 void TestWaitDiscriminativenessWithoutStealing() {
@@ -815,143 +783,6 @@ void TestMastersIsolation ( int p ) {
     }
 }
 
-//------------------------------------------------------------------------
-// Test for tbb::task::enqueue
-//------------------------------------------------------------------------
-
-const int PairsPerTrack = 100;
-
-class EnqueuedTask : public tbb::task {
-    task* my_successor;
-    int my_enqueue_order;
-    int* my_track;
-    tbb::task* execute() {
-        // Capture execution order in the very beginning
-        int execution_order = 2 - my_successor->decrement_ref_count();
-        // Create some local work.
-        TaskGenerator& p = *new( allocate_root() ) TaskGenerator(2,2);
-        spawn_root_and_wait(p);
-        if( execution_order==2 ) { // the "slower" of two peer tasks
-            ++nCompletedPairs;
-            // Of course execution order can differ from dequeue order.
-            // But there is no better approximation at hand; and a single worker
-            // will execute in dequeue order, which is enough for our check.
-            if (my_enqueue_order==execution_order)
-                ++nOrderedPairs;
-            FireTwoTasks(my_track);
-            destroy(*my_successor);
-        }
-        return NULL;
-    }
-public:
-    EnqueuedTask( task* successor, int enq_order, int* track )
-    : my_successor(successor), my_enqueue_order(enq_order), my_track(track) {}
-
-    // Create and enqueue two tasks
-    static void FireTwoTasks( int* track ) {
-        int progress = ++*track;
-        if( progress < PairsPerTrack ) {
-            task* successor = new (allocate_root()) tbb::empty_task;
-            successor->set_ref_count(2);
-            enqueue( *new (allocate_root()) EnqueuedTask(successor, 1, track) );
-            enqueue( *new (allocate_root()) EnqueuedTask(successor, 2, track) );
-        }
-    }
-
-    static tbb::atomic<int> nCompletedPairs;
-    static tbb::atomic<int> nOrderedPairs;
-};
-
-tbb::atomic<int> EnqueuedTask::nCompletedPairs;
-tbb::atomic<int> EnqueuedTask::nOrderedPairs;
-
-const int nTracks = 10;
-static int TaskTracks[nTracks];
-const int stall_threshold = 100000;
-
-void TimedYield( double pause_time );
-
-class ProgressMonitor {
-public:
-    void operator() ( ) {
-        int track_snapshot[nTracks];
-        int stall_count = 0, uneven_progress_count = 0, last_progress_mask = 0;
-        for(int i=0; i<nTracks; ++i)
-            track_snapshot[i]=0;
-        bool completed;
-        do {
-            // Yield repeatedly for at least 1 usec
-            TimedYield( 1E-6 );
-            int overall_progress = 0, progress_mask = 0;
-            const int all_progressed = (1<<nTracks) - 1;
-            completed = true;
-            for(int i=0; i<nTracks; ++i) {
-                int ti = TaskTracks[i];
-                int pi = ti-track_snapshot[i];
-                if( pi ) progress_mask |= 1<<i;
-                overall_progress += pi;
-                completed = completed && ti==PairsPerTrack;
-                track_snapshot[i]=ti;
-            }
-            // The constants in the next asserts are subjective and may need correction.
-            if( overall_progress )
-                stall_count=0;
-            else {
-                ++stall_count;
-                // no progress for at least 0.1 s; consider it dead.
-                ASSERT(stall_count < stall_threshold, "no progress on enqueued tasks; deadlock, or the machine is oversubsribed?");
-            }
-            if( progress_mask==all_progressed || progress_mask^last_progress_mask ) {
-                uneven_progress_count = 0;
-                last_progress_mask = progress_mask;
-            }
-            else if ( overall_progress > 2 ) {
-                ++uneven_progress_count;
-                ASSERT(uneven_progress_count < 5, "some enqueued tasks seem stalling; no simultaneous progress?");
-            }
-        } while( !completed );
-    }
-};
-
-void TestEnqueue( int p ) {
-    REMARK("testing task::enqueue for %d threads\n", p);
-    for(int mode=0;mode<3;++mode) {
-        tbb::task_scheduler_init init(p);
-        EnqueuedTask::nCompletedPairs = EnqueuedTask::nOrderedPairs = 0;
-        for(int i=0; i<nTracks; ++i) {
-            TaskTracks[i] = -1; // to accomodate for the starting call
-            EnqueuedTask::FireTwoTasks(TaskTracks+i);
-        }
-        ProgressMonitor pm;
-        tbb::tbb_thread thr( pm );
-        if(mode==1) {
-            // do some parallel work in the meantime
-            for(int i=0; i<10; i++) {
-                TaskGenerator& g = *new( tbb::task::allocate_root() ) TaskGenerator(2,5);
-                tbb::task::spawn_root_and_wait(g);
-                TimedYield( 1E-6 );
-            }
-        }
-        if( mode==2 ) {
-            // Additionally enqueue a bunch of empty tasks. The goal is to test that tasks
-            // allocated and enqueued by a thread are safe to use after the thread leaves TBB.
-            tbb::task* root = new (tbb::task::allocate_root()) tbb::empty_task;
-            root->set_ref_count(100);
-            for( int i=0; i<100; ++i )
-                tbb::task::enqueue( *new (root->allocate_child()) tbb::empty_task );
-            init.terminate(); // master thread deregistered
-        }
-        thr.join();
-        ASSERT(EnqueuedTask::nCompletedPairs==nTracks*PairsPerTrack, NULL);
-        ASSERT(EnqueuedTask::nOrderedPairs<EnqueuedTask::nCompletedPairs,
-            "all task pairs executed in enqueue order; de facto guarantee is too strong?");
-    }
-}
-
-//------------------------------------------------------------------------
-// Run all tests.
-//------------------------------------------------------------------------
-
 int TestMain () {
 #if TBB_USE_EXCEPTIONS
     TestUnconstructibleTask<1>();
@@ -966,7 +797,6 @@ int TestMain () {
         TestSpawnChildren( p );
         TestSpawnRootList( p );
         TestSafeContinuation( p );
-        TestEnqueue( p );
         TestLeftRecursion( p );
         TestDag( p );
         TestAffinity( p );
@@ -977,11 +807,3 @@ int TestMain () {
     }
     return Harness::Done;
 }
-
-#include "tbb/tick_count.h"
-void TimedYield( double pause_time ) {
-    tbb::tick_count start = tbb::tick_count::now();
-    while( (tbb::tick_count::now()-start).seconds() < pause_time )
-        __TBB_Yield();
-}
-
