@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,13 +26,17 @@
     the GNU General Public License.
 */
 
-//! task_handle<T> cannot be instantiated with a function ptr without explicit cast
-#define __TBB_FUNC_PTR_AS_TEMPL_PARAM_BROKEN ((__linux__ || __APPLE__) && __INTEL_COMPILER && __INTEL_COMPILER < 1100) || __SUNPRO_CC
-#define __TBB_UNQUALIFIED_CALL_OF_DTOR_BROKEN (__GNUC__==3 && __GNUC_MINOR__<=3)
+#include "harness_defs.h"
 
+//TODO: check why test fails with metro
+#include "tbb/tbb_config.h"
+#if !__TBB_WIN8UI_SUPPORT
 #ifndef TBBTEST_USE_TBB
     #define TBBTEST_USE_TBB 1
 #endif
+#else
+    #define TBBTEST_USE_TBB 0
+#endif
 
 #if !TBBTEST_USE_TBB
     #if defined(_MSC_VER) && _MSC_VER < 1600
@@ -204,7 +208,7 @@ atomic_t SharedGroupBodyImpl::s_tasksExecuted;
 
 class  SharedGroupBody : NoAssign, Harness::NoAfterlife {
     bool m_bOwner;
-    mutable SharedGroupBodyImpl *m_pImpl;
+    SharedGroupBodyImpl *m_pImpl;
 public:
     SharedGroupBody ( uint_t numThreads, uint_t sharingMode = 0 )
         : m_bOwner(true)
@@ -485,7 +489,7 @@ void TestFibWithLambdas () {
 //------------------------------------------------------------------------
 
 void TestFibWithMakeTask () {
-    REMARK ("Make_task test");
+    REMARK ("Make_task test\n");
     atomic_t sum;
     sum = 0;
     Concurrency::task_group rg;
@@ -680,12 +684,12 @@ void StructuredLaunchChildren () {
     count = 0;
     Concurrency::structured_task_group g;
     bool exceptionCaught = false;
-    typedef Concurrency::task_handle<ThrowingTask> handle_type;
-    static const unsigned hSize = sizeof(handle_type);
+    typedef Concurrency::task_handle<ThrowingTask> throwing_handle_type;
+    static const unsigned hSize = sizeof(throwing_handle_type);
     char handles[NUM_CHORES * hSize];
     for( unsigned i = 0; i < NUM_CHORES; ++i ) {
-        handle_type *h = (handle_type*)(handles + i * hSize);
-        new ( h ) handle_type( ThrowingTask(count) );
+        throwing_handle_type *h = (throwing_handle_type*)(handles + i * hSize);
+        new ( h ) throwing_handle_type( ThrowingTask(count) );
         g.run( *h );
     }
     __TBB_TRY {
@@ -705,7 +709,7 @@ void StructuredLaunchChildren () {
     } CATCH_ANY();
     ASSERT( !g_Throw || exceptionCaught, "No exception in the child task group" );
     for( unsigned i = 0; i < NUM_CHORES; ++i )
-        ((handle_type*)(handles + i * hSize))->~handle_type();
+        ((throwing_handle_type*)(handles + i * hSize))->~throwing_handle_type();
     if ( g_Rethrow && g_ExceptionCount > SKIP_GROUPS ) {
 #if __TBB_SILENT_CANCELLATION_BROKEN
         g_CancellationPropagationInProgress = true;
